unit MainUnit;

{$mode objfpc}{$H+}

{*******************************************************************************
*
*                       Demo Application for midi.pas
*
*                Created as 64 Bit Lazarus Project on 2020-12-24
*
*
*                Latest changes: 2020-12-26
*******************************************************************************}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  Messages, Midi;

type
  { TForm1 }
  TForm1 = class(TForm)
    ButtonClose: TButton;
    ButtonShowDevices: TButton;
    ButtonOpenCloseMIDI: TButton;
    Memo1: TMemo;
    Panel1: TPanel;
    Panel2: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var {%H-}CloseAction: TCloseAction);
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonShowDevicesClick(Sender: TObject);
    procedure ButtonOpenCloseMIDIClick(Sender: TObject);
  private
    MidiIsOpen:boolean; // a var for the "ButtonOpenCloseMIDI"

    // This is the EVENT method, as a "Procedure Of Object":
    procedure OnMidiInData( const aDeviceIndex: integer; const aStatus, aData1, aData2: byte);

    // Use "windows message" to do something in GUI :
    procedure WmMidiDataArrived( var Msg: TMessage); message WM_MIDIDATA_ARRIVED;
    //procedure WmMidiInError( var Msg: TMessage); message WM_MIM_ERROR;
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses JWAwindows;



{ *** TForm1 ***************************************************************** }
procedure TForm1.FormCreate(Sender: TObject);
begin
  // It is NOT REQUIRED to create MIDI IN and OUT Objects ..
  //
  // Please see comments in midi.pas, behind "implementation"

  // ---------------------------------------------------------
  // This is how to assign the "Procedure Of Object"
  // to the "EVENT" in TMidiInput :
  MidiInput.OnMidiData:= @Self.OnMidiInData;
  MidiIsOpen:= FALSE;
end;


procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  // avoid an exception on Close - close all MIDI devices :
  if MidiIsOpen = TRUE then Self.ButtonOpenCloseMIDIClick( Self);
end;


// This code is executed in a CALLBACK PRODECURE,
// linked in as a "Procedure Of Object" :
procedure TForm1.OnMidiInData( const aDeviceIndex: integer; const aStatus, aData1, aData2: byte);
begin
  // Here we immediately send "MIDI In" values to corresponding MIDI out
  //
  // Hint: usually "Microsoft GS Wavetable Synth" is present on every Windows PC
  // and is only a "MIDI Out" device, so for this DEMO program we shift aDeviceIndex +1 ..

  if (aStatus <> $F8)     // IGNORE real-time message clock $F8 = 248
    and (aStatus <> $FE)  // IGNORE "Active Sensing" $FE = 254
    then MidiOutput.Send( aDeviceIndex +1, aStatus, aData1, aData2);


  // To display the values, or anything else less time crtitcal,
  // post the MIDI data (for example) to a MainForm procedure
  //
  // Because:  NEVER execute GUI code inside a CALLBACK PRODECURE !!!
  PostMessage( Application.MainForm.Handle,
               WM_MIDIDATA_ARRIVED,
               aDeviceIndex,
               aStatus
               + (aData1 and $ff) shl 8
               + (aData2 and $FF) shl 16 );
end;


// Process "windows message":
procedure TForm1.WmMidiDataArrived(var Msg: TMessage); // message WM_MIDIDATA_ARRIVED;
begin
  // simply display the values:
  if (Msg.lParamlo <> $F8)     // IGNORE real-time message clock $F8 = 248
    and (Msg.lParamlo <> $FE)  // IGNORE "Active Sensing" $FE = 254
    then Memo1.Append( IntToStr( Msg.wParamhi) +' '
                       +IntToStr( Msg.wParamlo) +'  '
                       // MIDI Note on / off
                       +IntToHex( Msg.lParamlo and $FF, 2) +' '
                       // MIDI Note value
                       +IntToHex( (Msg.lParamlo shr 8) and $FF, 2) +' '
                       // MIDI Key On Velocity
                       +IntToHex( Msg.lParamhi and $FF, 2)
                       );

                       // this value always contains NULL and can be ignored :
                       //+IntToHex( (Msg.lParamhi shr 8) and $FF, 2)
end;


{ not in use in this Demo Project:
procedure TForm1.WmMidiInError(var Msg: TMessage); // message WM_MIM_ERROR;
begin
  //
end;
}


{ *** Buttons **************************************************************** }
procedure TForm1.ButtonCloseClick(Sender: TObject);
begin
  Self.close;
end;


procedure TForm1.ButtonShowDevicesClick(Sender: TObject);
var i:integer;
begin
  // fill memo with strings
  Memo1.Append( 'List of available MIDI outputs:');
  Memo1.Append( '-------------------------------');

  for i:= 0 to MidiOutput.Devices.Count -1 do
    begin
      Memo1.Append( IntToStr( i) +' - ' +MidiOutput.Devices[i]);
    end;
  Memo1.Append( '');

  // fill memo with strings
  Memo1.Append( 'List of available MIDI inputs:');
  Memo1.Append( '------------------------------');

  for i:= 0 to MidiInput.Devices.Count -1 do
    begin
      Memo1.Append( IntToStr( i) +' - ' +MidiInput.Devices[i]);
    end;
  Memo1.Append( '');
end;


procedure TForm1.ButtonOpenCloseMIDIClick(Sender: TObject);
var i:integer;
begin
  if MidiIsOpen = FALSE
    then
      // Open all MIDI inputs
      begin
        MidiIsOpen:= TRUE;
        ButtonClose.Enabled:= FALSE;  // DO NOT close app while MIDI in is active !
        ButtonOpenCloseMIDI.Caption:= 'Close all Devices';
        Memo1.Append( 'Opened all available MIDI devices');

        for i:= 0 to MidiOutput.Devices.Count -1 do
          begin
            MidiOutput.Open( i);
          end;

        for i:= 0 to MidiInput.Devices.Count -1 do
          begin
            MidiInput.Open( i);
          end;
      end

    else
      // Close all MIDI inputs
      begin
        for i:= 0 to MidiInput.Devices.Count -1 do
          begin
            MidiInput.Close( i);
          end;

        for i:= 0 to MidiOutput.Devices.Count -1 do
          begin
            MidiOutput.Close( i);
          end;

        MidiIsOpen:= FALSE;
        ButtonClose.Enabled:= TRUE;
        ButtonOpenCloseMIDI.Caption:= 'Open all Devices';
        Memo1.Append( 'Closed all available MIDI devices');
      end;
end;


end.

